#! /usr/bin/env python
# -*- coding: koi8-r -*-

from __future__ import print_function
from ..lazy.dict import LazyDictInitFunc

#
# Rus -> Lat transliteration (koi2lat and win2lat)
#

koi2lat_d = {
   "": "A",
   "": "B",
   "": "V",
   "": "G",
   "": "D",
   "": "E",
   "": "Zh",
   "": "Z",
   "": "I",
   "": "Y",
   "": "K",
   "": "L",
   "": "M",
   "": "N",
   "": "O",
   "": "P",
   "": "R",
   "": "S",
   "": "T",
   "": "U",
   "": "F",
   "": "H",
   "": "Ts",
   "": "Ch",
   "": "Sh",
   "": "Sh",
   "": "'",
   "": "'",
   "": "Y",
   "": "E",
   "": "Yu",
   "": "Ya",
   "": "a",
   "": "b",
   "": "v",
   "": "g",
   "": "d",
   "": "e",
   "": "zh",
   "": "z",
   "": "i",
   "": "y",
   "": "k",
   "": "l",
   "": "m",
   "": "n",
   "": "o",
   "": "p",
   "": "r",
   "": "s",
   "": "t",
   "": "u",
   "": "f",
   "": "h",
   "": "ts",
   "": "ch",
   "": "sh",
   "": "sh",
   "": "'",
   "": "'",
   "": "y",
   "": "e",
   "": "yu",
   "": "ya",
}

def make_xxx2lat(encoding="cp1251"):
   d = {}
   for k, v in koi2lat_d.items():
      d[k] = v
   return d


win2lat_d = LazyDictInitFunc(make_xxx2lat, encoding="cp1251")


def rus2lat(instr, rus2lat_d = koi2lat_d):
   out = []
   for c in instr:
      c = rus2lat_d.get(c, c)
      if isinstance(c, int):
         c = chr(c)
      out.append(c)
   return ''.join(out)


koi2lat = rus2lat

def win2lat(instr):
   return rus2lat(instr, win2lat_d)


if __name__ == "__main__":
   Test = "  .  xyz   "
   print("Test:", Test)
   print(":", koi2lat(Test))
   print(":", win2lat(Test))
